@@ -16,6 +16,9 @@
 #**** along with this program.  If not, see <http://www.gnu.org/licenses/> ****
 #******************************************************************************
 
+"""
+ IMPORTS
+"""
 import sys
 import ConfigParser
 
@@ -31,7 +34,7 @@ import playerfactory as PlayerFactory
 import mediasong as MediaSong
 
 # The current version of Kap
-_VERSION = "0.1.0"
+_VERSION = "0.2.0"
 
 
 class Kap(wx.Frame):
@@ -71,8 +74,12 @@ class Kap(wx.Frame):
     __playContinuous = False
     # Indicates the programme should randomly play a song from the play-list.
     __playRandom = False
+    # Indicates the current song should be removed from the play-list
+    __removeFromPlayList = True
     # Additional arguments that should be passed on the the player.
     __additionalPlayerArguements = ""
+    # Holds the current playing song
+    __currentSong = None
 
     """
      EVENT ID
@@ -157,6 +164,9 @@ class Kap(wx.Frame):
             if self.__sfCnfgPrsrKap.has_option('Kap', "searcRefreshRate"):
                 self.__searchRefreshRate = \
                     self.__sfCnfgPrsrKap.getint("Kap", "searcRefreshRate")
+            if self.__sfCnfgPrsrKap.has_option('Kap', "removeFromPlayList"):
+                self.__removeFromPlayList = \
+                    self.__sfCnfgPrsrKap.getboolean("Kap", "removeFromPlayList")
         if self.__sfCnfgPrsrKap.has_section('Player'):
             if self.__sfCnfgPrsrKap.has_option('Player', \
                     "additionalPlayerArguements"):
@@ -218,6 +228,8 @@ class Kap(wx.Frame):
         self.__sfCnfgPrsrKap.set('Kap', 'windowSize', "%s" % self.GetSize())
         self.__sfCnfgPrsrKap.set('Kap', 'searchRefreshRate', "%s" % \
             self.__searchRefreshRate)
+        self.__sfCnfgPrsrKap.set('Kap', 'removeFromPlayList', "%s" % \
+            self.__removeFromPlayList)
 
     def __createMenus(self):
         """ Create the file menu options.
@@ -312,6 +324,8 @@ class Kap(wx.Frame):
             self.__onAddSongToPlayList)
         # Player Window Events
         wx.EVT_BUTTON(self, self.__playerWindow.ID_PLAY, self.__onPlaySong)
+        wx.EVT_BUTTON(self, self.__playerWindow.ID_RESTART, \
+            self.__onRestartSong)
         wx.EVT_BUTTON(self, self.__playerWindow.ID_STOP, self.__onStopSong)
         wx.EVT_COMMAND_SCROLL(self, self.__playerWindow.ID_VOLUME, \
             self.__onChangeVolume)
@@ -375,7 +389,7 @@ class Kap(wx.Frame):
         """
         kd = KapDialogue.KapDialogue(self, \
             self.__additionalPlayerArguements, self.__trackRefreshRate, \
-                self.__searchRefreshRate)
+                self.__searchRefreshRate, self.__removeFromPlayList)
         kd.ShowModal()
         if kd.blIsSettingsDirty:
             # Update the player arguments
@@ -390,6 +404,7 @@ class Kap(wx.Frame):
             self.__trackTimer.Stop()
             self.__trackTimer.Start(self.__trackRefreshRate)
             self.__searchRefreshRate = kd.searchRefreshRate()
+            self.__removeFromPlayList = kd.removeFromPlayList()
 
     def __onFindSong(self, event):
         """ Starts the database searching for songs.
@@ -398,22 +413,30 @@ class Kap(wx.Frame):
            window.
          wx.Event event: is the event object (Not Used)
         """
-        if not self.__searchTimer.IsRunning():
-            self.StatusBar.SetStatusText("Searching For Media...", \
-                self.__GENERAL_STATUS_FIELD)
-            self.__database.findSongs( \
-                self.__searchWindow.searchTerms(), \
-                self.__searchWindow.searchForAudios(), \
-                self.__searchWindow.searchForVideos())
-            self.__searchTimer.Start(self.__searchRefreshRate)
+        if self.__searchTimer.IsRunning():
+            self.__cancelSearch()
+
+        self.StatusBar.SetStatusText("Searching For Media...", \
+            self.__GENERAL_STATUS_FIELD)
+        self.__database.findSongs( \
+            self.__searchWindow.searchTerms(), \
+            self.__searchWindow.searchForAudios(), \
+            self.__searchWindow.searchForVideos())
+        self.__searchTimer.Start(self.__searchRefreshRate)
 
     def __onCancelFindSong(self, event):
         """ Stops a user library search.
 
          wx.Event event: is the event object (Not Used)
         """
-        self.__searchTimer.Stop()
         self.__searchWindow.clearSearchTerms()
+        self.__cancelSearch()
+
+    def __cancelSearch(self):
+        """ Stops a user library search. """
+        self.__searchTimer.Stop()
+        self.StatusBar.SetStatusText("Search Cancelled", \
+            self.__GENERAL_STATUS_FIELD)
 
     def __updateSearchResults(self, event):
         """ Updates the search results window.
@@ -488,7 +511,16 @@ class Kap(wx.Frame):
         else:
             self.__handleStartSong()
 
-    def __handleStartSong(self):
+    def __onRestartSong(self, event):
+        """ Restarts the current playing song. """
+        if self.__playerFactory.player.isPlaying() or \
+                self.__playerFactory.player.isPaused():
+            currentSong = self.__currentSong
+            self.__playerFactory.player.stopSong()
+            self.__handleEndSong()
+            self.__handleStartSong(currentSong)
+
+    def __handleStartSong(self, song=None):
         """ Handles the details of starting of a song.
 
          Selects a song from the play-list based on the current settings and
@@ -496,14 +528,20 @@ class Kap(wx.Frame):
            The song is selected either by direct input from the user, by
            random, or by continuous play from the top of the play-list. Also,
            sets the player interface to reflect the current state.
+         MediaSong song: is the specific song to play, else pulls from the
+           play-list.
         """
         # Determine which song to play
-        if self.__playRandom:
-            song = self.__playListWindow.playRandomSong()
-        elif self.__playContinuous:
-            song = self.__playListWindow.playFirstSong()
-        else:
-            song = self.__playListWindow.playSelectedSong()
+        if song is None:
+            if self.__playRandom:
+                song = self.__playListWindow.playRandomSong( \
+                    self.__removeFromPlayList)
+            elif self.__playContinuous:
+                song = self.__playListWindow.playFirstSong( \
+                    self.__removeFromPlayList)
+            else:
+                song = self.__playListWindow.playSelectedSong( \
+                    self.__removeFromPlayList)
 
         # Try to play the song
         if song.filePath is not None:
@@ -512,11 +550,13 @@ class Kap(wx.Frame):
                 self.__playerWindow.togglePlay()
                 self.StatusBar.SetStatusText("Loaded: %s - %s" % (song.title, \
                     song.artist), self.__PLAYER_STATUS_FIELD)
+                self.__currentSong = song
             else:
                 wx.MessageBox("Could not start player or load media file!", \
                     "Player Error", wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP)
-                if song.filePath:
+                if song.filePath and not self.__removeFromPlayList:
                     self.__playListWindow.addSongsToPlayList([song])
+                self.__currentSong = None
 
     def __onStopSong(self, event):
         """ Stops the currently playing song.
@@ -580,6 +620,7 @@ class Kap(wx.Frame):
             self.__playerWindow.togglePlay()
         self.StatusBar.SetStatusText("Player Ready", \
             self.__PLAYER_STATUS_FIELD)
+        self.__currentSong = None
 
     def __onEndSong(self, event):
         """ Handles the end of a song event.
@@ -613,12 +654,13 @@ class Kap(wx.Frame):
          wx.Event event: is the event object (Not Used)
         """
         abtnfAbout = wx.AboutDialogInfo()
-        abtnfAbout.SetName("Karaoke Audio-video Player")
-        abtnfAbout.SetVersion(_VERSION)
-        abtnfAbout.AddDeveloper("John Schneiderman <JohnMS@member.fsf.org>")
+        #abtnfAbout.AddArtist("N/A")
         abtnfAbout.SetCopyright("(C) 2009 John Schneiderman")
         abtnfAbout.SetDescription( \
             "A player with a search-able media library.")
+        abtnfAbout.AddDeveloper("John Schneiderman <JohnMS@member.fsf.org>")
+        #abtnfAbout.AddDocWriter("N/A")
+        #abtnfAbout.SetIcon(icon)
         GPLv3_Notice = "Kap is free software; you can redistribute it and/or"
         GPLv3_Notice += "\nmodify it under the terms of the GNU General "
         GPLv3_Notice += "Public\nLicense as published by the Free Software "
@@ -634,6 +676,10 @@ class Kap(wx.Frame):
         GPLv3_Notice += " Software\nFoundation, Inc., 59 Temple Place, Suite"
         GPLv3_Notice += " 330, Boston, MA  02111-1307  USA"
         abtnfAbout.SetLicence(GPLv3_Notice)
+        abtnfAbout.SetName("Karaoke Audio-video Player")
+        #abtnfAbout.AddTranslator("N/A")
+        abtnfAbout.SetVersion(_VERSION)
+        abtnfAbout.SetWebSite("http://kap.nongnu.org")
         wx.AboutBox(abtnfAbout)
 
     def __onMenuPlayer(self, event):
